package com.iden.bms.face;




import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.face.monitor.FaceMonitor;
import com.face.monitor.model.FaceModel;
import com.iden.bms.service.PersonService;
import com.iden.common.entity.IdenCamera;
import com.iden.common.entity.IdenCommunity;
import com.iden.common.entity.IdenFaceImage;

import com.iden.common.entity.IdenPerson;
import com.iden.common.enums.PersonTypeEnum;
import com.iden.common.service.IdenCameraService;
import com.iden.common.service.IdenCommunityService;
import com.iden.common.service.IdenFaceImageService;

import com.iden.common.util.ByteUtil;
import com.iden.common.util.DateUtils;
import com.iden.common.util.FileUtil;
import com.iden.common.vo.FaceRetrieveResultVO;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.math.BigDecimal;
import java.util.*;


/**
 * @Author: stw
 * @Date: 2021/7/14
 * @Desc:
 */
@Service
public class FaceIdenService {
    @Resource
    private IdenFaceImageService idenFaceImageService;
    @Resource
    private IdenCameraService idenCameraService;
    @Resource
    private IdenCommunityService idenCommunityService;
    @Resource
    private PersonService personService;


    @Value("${iden.root:#{null}}")
    private String idenRoot;
    @Value("${image.url:#{null}}")
    private String imageUrl;

    private static final Logger logger = LogManager.getLogger(FaceIdenService.class);

    /**
     *  处理各摄像头上传的图像，去除模糊的、没有人脸的、连拍的图像
     * @throws Exception
     */
    public void handleCameraImage() throws Exception {
        logger.info("FaceIdenService.handleCameraImage start...");

        File originImageDirFile = new File(idenRoot + "data/origin/camera/image");

        //使用摄像头编码做目录名，里面图像或视频的名称上加上拍照时间，
        // 比如  20211217123023.jpg     20211217123023~20211217123323.mp4
        if (originImageDirFile.isDirectory()) {
            File[] cameraCodeDirs = originImageDirFile.listFiles();
            if(cameraCodeDirs != null && cameraCodeDirs.length > 0){

                //初始化引擎，加载人员图像人脸特征库
                String personImageDir = idenRoot + "data/final/person";
                File[] imgPersonFiles = new File(personImageDir).listFiles();
                FaceMonitor faceMonitorPerson = FaceIdenTool.initFaceMonitor(idenRoot,imgPersonFiles);

                for(File cameraCodedir : cameraCodeDirs) {
                    if(cameraCodedir.isDirectory()){
                        File[] imgFiles = FileUtil.sortByName(cameraCodedir.listFiles());
                        if(imgFiles != null && imgFiles.length > 0){
                            String cameraCode = cameraCodedir.getName();
                            QueryWrapper<IdenCamera> queryWrapper = new QueryWrapper<>();
                            queryWrapper.lambda().eq(IdenCamera::getCode,cameraCode);
                            IdenCamera idenCamera = idenCameraService.getOne(queryWrapper);
                            if(idenCamera == null){
                                continue;
                            }

                           //获取特征码
                            FaceModel[] faceModels = FaceIdenTool.extractFeature(idenRoot,imgFiles);

                            List<IdenFaceImage> idenFaceImageList = new ArrayList<>();
                            for(int i = 0; i < imgFiles.length; i++) {
                                File imgFile  = imgFiles[i];
                                IdenFaceImage idenFaceImage = new IdenFaceImage();
                                idenFaceImage.setCameraId(idenCamera.getId());
                                idenFaceImage.setCommunityId(idenCamera.getCommunityId());
                                String photographPlace = idenCamera.getPlace();
                                if (StringUtils.isEmpty(photographPlace)) {
                                    IdenCommunity idenCommunity = idenCommunityService.getById(idenCamera.getCommunityId());
                                    if(idenCommunity != null) {
                                        photographPlace = idenCommunity.getName() + "(" + idenCamera.getLongitude() + ":" + idenCamera.getLatitude() + ")";
                                    }
                                }
                                idenFaceImage.setPhotographPlace(photographPlace);
                                idenFaceImage.setLongitude(idenCamera.getLongitude());
                                idenFaceImage.setLatitude(idenCamera.getLatitude());
                                String imgFileName = imgFile.getName();
                                String photographTime = imgFileName.substring(0, imgFileName.indexOf("_"));
                                idenFaceImage.setPhotographTime(DateUtils.strToDate(photographTime,"yyyyMMddHHmmss"));

                                //识别特征码
                                if(faceModels[i] != null){
                                    idenFaceImage.setFeatPtr(ByteUtil.byte2Hex(faceModels[i].getFeatValue()));
                                }

                                File finalDir = new File(imgFile.getParentFile().getAbsolutePath().replace("origin","final"));
                                if(!finalDir.exists()){
                                    finalDir.mkdirs();
                                }
                                logger.info("FaceIdenService.handleCameraImage ...finalDir == " + finalDir.getAbsolutePath());
                                logger.info("FaceIdenService.handleCameraImage ...imgFile.getName() == " + imgFileName);
                                File finalImgFile = new File(finalDir, imgFileName);

                                logger.info("FaceIdenService.handleCameraImage ...finalImgFile == " + finalImgFile.getAbsolutePath());

                                imgFile.renameTo(finalImgFile);//移动到最终目录

                                idenFaceImage.setImage( imageUrl + cameraCode + "/" + finalImgFile.getName());

                                //和人员图像库比对特征码，关联personId
                                if(faceModels[i] != null){

                                    FaceRetrieveResultVO faceRetrieveResultVO = FaceIdenTool.getHitResult(faceMonitorPerson,faceModels[i]);
                                    if (faceRetrieveResultVO != null){
                                        int hitIndex = faceRetrieveResultVO.getIndex();
                                        IdenPerson idenPerson = personService.getPersonIdByImageFileName(imgPersonFiles[hitIndex].getName());
                                        if(idenPerson != null) {
                                            idenFaceImage.setPersonId(idenPerson.getId());
                                            idenFaceImage.setType(idenPerson.getType());
                                            idenFaceImage.setGender(idenPerson.getGender());
                                            float score = faceRetrieveResultVO.getScore();
                                            BigDecimal b = new BigDecimal(score);
                                            double similarity = b.divide(new BigDecimal(100)).setScale(4,BigDecimal.ROUND_HALF_UP).doubleValue();

                                            idenFaceImage.setSimilarity(similarity);
                                        }
                                    }

                                }
                                if (idenFaceImage.getPersonId() == null){
                                    idenFaceImage.setType(PersonTypeEnum.STRANGER.getValue());
                                }
                                idenFaceImage.setCreateTime(new Date());
                                idenFaceImageList.add(idenFaceImage);
                            }

                            //把图像存到图库中
                            if (CollUtil.isNotEmpty(idenFaceImageList)) {
                                idenFaceImageService.saveBatch(idenFaceImageList);
                            }

                        }
                    }
                }
                //释放人脸引擎
                FaceIdenTool.releaseEngine(faceMonitorPerson);
            }
        }
        logger.info("FaceIdenService.handleCameraImage end");
    }


    /**
     *
     * @throws Exception
     */

    public void handleCameraVideo() throws Exception {

    }

}
