package com.iden.bms.schedule;



import com.iden.bms.service.WarningService;
import com.iden.common.cache.RedisKeyConstant;
import com.iden.common.cache.RedisUtil;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.util.UUID;

/**
 * 把预警人员的轨迹入库
 */
@Configuration
@EnableScheduling
@EnableAsync
public class WarningSchedule {

    @Resource
    private WarningService warningService;
    @Resource
    private RedisUtil redisUtil;

    /**
     * 处理预警
     */
    @Async
    @Scheduled(cron = "20 0/5 * * * ?")
    public void handleWarning() {
        String key = RedisKeyConstant.HANDLE_WARNING;
        String requestId = UUID.randomUUID().toString();
        boolean result = redisUtil.tryLock(key,requestId,5 * 60);
        try {
            if (result) {
                warningService.handleWarning();
            }
        } catch (Exception e) {
            e.getMessage();
        }  finally {
            redisUtil.releaseLock(key,requestId);
        }
    }


}
