package com.iden.bms.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.iden.common.entity.IdenCommunity;

import com.iden.common.service.IdenCommunityService;
import com.iden.common.util.MyBeanUtils;
import com.iden.common.vo.CommunityVO;
import com.iden.common.vo.PageReqVO;
import com.iden.common.vo.UserLoginedConvertVO;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class CommunityService {

    @Resource
    private IdenCommunityService idenCommunityService;


    public Integer countCommunity(String district, String subdistrict, String name, UserLoginedConvertVO loginUser) {
        QueryWrapper<IdenCommunity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(StrUtil.isNotEmpty(name),IdenCommunity::getName,name)
                .eq(StrUtil.isNotEmpty(district),IdenCommunity::getDistrict,district)
                .eq(StrUtil.isNotEmpty(subdistrict),IdenCommunity::getSubdistrict,subdistrict);
        return this.idenCommunityService.count(queryWrapper);

    }

    /**
     * 查询小区列表
     * @return
             */
    public IPage<CommunityVO> listCommunity(String district, String subdistrict, String name, UserLoginedConvertVO loginUser, PageReqVO pageReqVo) {
            IPage<IdenCommunity> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
            QueryWrapper<IdenCommunity> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().like(StrUtil.isNotEmpty(name),IdenCommunity::getName,name)
                    .eq(StrUtil.isNotEmpty(district),IdenCommunity::getDistrict,district)
                    .eq(StrUtil.isNotEmpty(subdistrict),IdenCommunity::getSubdistrict,subdistrict)
                    .orderByAsc(IdenCommunity::getCode);

            IPage<IdenCommunity> pageRes = this.idenCommunityService.page(page, queryWrapper);
            IPage<CommunityVO> results = new Page<>(pageRes.getCurrent(),pageRes.getSize(),pageRes.getTotal());
            if(CollUtil.isNotEmpty(pageRes.getRecords())){
                List<CommunityVO> list = new ArrayList<>();
                pageRes.getRecords().forEach(item -> {
                    CommunityVO resVO = new CommunityVO();
                    BeanUtils.copyProperties(item,resVO);
                    list.add(resVO);
                });
                results.setRecords(list);
            }
            return results;
    }

    /**
     * 删除小区
     * @param id
     * @return
     */
    public boolean deleteById(Long id){
        return this.idenCommunityService.removeById(id);
    }

    /**
     * 详情
     * @param id
     * @return
     */
    public CommunityVO getCommunityById(Long id){
        IdenCommunity idenCommunity = this.idenCommunityService.getById(id);
        if (idenCommunity!=null){
            CommunityVO resVO = new CommunityVO();
            BeanUtil.copyProperties(idenCommunity,resVO);
            return resVO;
        }
        return null;
    }


    /**
     * 保存小区
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public int createCommunity(CommunityVO vo, UserLoginedConvertVO loginUser){
        QueryWrapper<IdenCommunity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(IdenCommunity::getName,vo.getName());
        if(idenCommunityService.count(queryWrapper) > 0){
            return 1;
        }
        QueryWrapper<IdenCommunity> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.lambda().eq(IdenCommunity::getCode,vo.getCode());
        if(idenCommunityService.count(queryWrapper2) > 0){
            return 2;
        }
        //保存小区
        IdenCommunity idenCommunity = new IdenCommunity();
        BeanUtil.copyProperties(vo,idenCommunity);
        idenCommunity.setCreateTime(new Date());

        this.idenCommunityService.save(idenCommunity);
        return 0;
    }

    /**
     * 修改小区
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public void updateCommunity(CommunityVO vo){
        //保存小区
        IdenCommunity idenCommunity = this.idenCommunityService.getById(vo.getId());
        MyBeanUtils.copyProperties(vo,idenCommunity);
        idenCommunity.setModifyTime(new Date());
        this.idenCommunityService.updateById(idenCommunity);

    }

}