package com.iden.bms.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.iden.bms.face.FaceIdenTool;
import com.iden.common.entity.IdenCrowd;
import com.iden.common.entity.IdenPerson;

import com.iden.common.entity.IdenPersonCrowdRef;
import com.iden.common.enums.PersonTypeEnum;
import com.iden.common.enums.PopulationTypeEnum;
import com.iden.common.exception.BDException;
import com.iden.common.service.IdenCrowdService;
import com.iden.common.service.IdenPersonCrowdRefService;
import com.iden.common.service.IdenPersonService;
import com.iden.common.util.DateUtils;
import com.iden.common.util.ImgUtil;
import com.iden.common.util.MyBeanUtils;
import com.iden.common.vo.PersonStaVO;
import com.iden.common.vo.PersonVO;
import com.iden.common.vo.PageReqVO;
import com.iden.common.vo.UserLoginedConvertVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileOutputStream;
import java.util.*;

/**
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class PersonService {

    @Resource
    private IdenPersonService idenPersonService;

    @Resource
    private IdenPersonCrowdRefService idenPersonCrowdRefService;
    @Resource
    private IdenCrowdService idenCrowdService;

    @Value("${iden.root:#{null}}")
    private String idenRoot;
    @Value("${file.url:#{null}}")
    private String fileUrl;


    public PersonStaVO getPersonStaVO(String type, String nameOrCred, String district, String subdistrict, Long communityId, Long crowdId, String address, String gender, String populationType, UserLoginedConvertVO loginUser) {
        QueryWrapper<IdenPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(IdenPerson::getType,type)
                .like(StrUtil.isNotEmpty(address), IdenPerson::getAddress,address)
                .eq(StrUtil.isNotEmpty(gender),IdenPerson::getGender,gender)
                .eq(StrUtil.isNotEmpty(populationType),IdenPerson::getPopulationType,populationType)
                .eq(StrUtil.isNotEmpty(district),IdenPerson::getDistrict,district)
                .eq(StrUtil.isNotEmpty(subdistrict),IdenPerson::getSubdistrict,subdistrict)
                .eq(communityId != null,IdenPerson::getCommunityId,communityId)
                .and(StrUtil.isNotEmpty(nameOrCred),wrapper -> wrapper.like(IdenPerson::getName,nameOrCred)
                        .or().like(IdenPerson::getCredentialsCode,nameOrCred));
         Integer personCount = this.idenPersonService.count(queryWrapper);

        QueryWrapper<IdenPerson> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.lambda().like(IdenPerson::getType,type)
                .like(StrUtil.isNotEmpty(address), IdenPerson::getAddress,address)
                .eq(StrUtil.isNotEmpty(gender),IdenPerson::getGender,gender)
                .eq(StrUtil.isNotEmpty(populationType),IdenPerson::getPopulationType,populationType)
                .eq(StrUtil.isNotEmpty(district),IdenPerson::getDistrict,district)
                .eq(StrUtil.isNotEmpty(subdistrict),IdenPerson::getSubdistrict,subdistrict)
                .eq(communityId != null,IdenPerson::getCommunityId,communityId)
                .and(StrUtil.isNotEmpty(nameOrCred),wrapper -> wrapper.like(IdenPerson::getName,nameOrCred)
                        .or().like(IdenPerson::getCredentialsCode,nameOrCred))
                .isNotNull(IdenPerson::getImage);
        Integer faceImageCount = this.idenPersonService.count(queryWrapper1);

        QueryWrapper<IdenPerson>  queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.select("distinct community_id")
                .lambda().like(IdenPerson::getType,type)
                .like(StrUtil.isNotEmpty(address), IdenPerson::getAddress,address)
                .eq(StrUtil.isNotEmpty(gender),IdenPerson::getGender,gender)
                .eq(StrUtil.isNotEmpty(populationType),IdenPerson::getPopulationType,populationType)
                .eq(StrUtil.isNotEmpty(district),IdenPerson::getDistrict,district)
                .eq(StrUtil.isNotEmpty(subdistrict),IdenPerson::getSubdistrict,subdistrict)
                .eq(communityId != null,IdenPerson::getCommunityId,communityId)
                .and(StrUtil.isNotEmpty(nameOrCred),wrapper -> wrapper.like(IdenPerson::getName,nameOrCred)
                        .or().like(IdenPerson::getCredentialsCode,nameOrCred));

        Integer communityCount = this.idenPersonService.count(queryWrapper2);

        PersonStaVO vo = new PersonStaVO();
        vo.setCommunityCount(communityCount);
        vo.setFaceImageCount(faceImageCount);
        vo.setPersonCount(personCount);
        return vo;
    }

    /**
     * 查询人员列表
     * @return
             */
    public IPage<PersonVO> listPerson(String type, String nameOrCred, String district, String subdistrict, Long communityId, Long crowdId, String address, String gender, String populationType, UserLoginedConvertVO loginUser, PageReqVO pageReqVo) {
            IPage<IdenPerson> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());

            QueryWrapper<IdenPerson> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().like(IdenPerson::getType,type)
                    .like(StrUtil.isNotEmpty(address), IdenPerson::getAddress,address)
                    .eq(StrUtil.isNotEmpty(gender),IdenPerson::getGender,gender)
                    .eq(StrUtil.isNotEmpty(populationType),IdenPerson::getPopulationType,populationType)
                    .eq(StrUtil.isNotEmpty(district),IdenPerson::getDistrict,district)
                    .eq(StrUtil.isNotEmpty(subdistrict),IdenPerson::getSubdistrict,subdistrict)
                    .eq(communityId != null,IdenPerson::getCommunityId,communityId)
                    .and(StrUtil.isNotEmpty(nameOrCred),wrapper -> wrapper.like(IdenPerson::getName,nameOrCred)
                    .or().like(IdenPerson::getCredentialsCode,nameOrCred))
                    .orderByDesc(IdenPerson::getModifyTime)
                    .orderByDesc(IdenPerson::getCreateTime);

            if (crowdId != null) {
                queryWrapper.apply(" in ( select person_id from iden_person_crowd_ref ipcr where ipcr.crowd_id = "+ crowdId + ")");
            }

            IPage<IdenPerson> pageRes = this.idenPersonService.page(page, queryWrapper);
            IPage<PersonVO> results = new Page<>(pageRes.getCurrent(),pageRes.getSize(),pageRes.getTotal());
            if(CollUtil.isNotEmpty(pageRes.getRecords())){
                List<PersonVO> list = new ArrayList<>();
                pageRes.getRecords().forEach(item -> {
                    PersonVO resVO = new PersonVO();
                    BeanUtils.copyProperties(item,resVO);
                    resVO.setPopulationTypeName(PopulationTypeEnum.getValueToName(resVO.getPopulationType()));
                    resVO.setTypeName(PersonTypeEnum.getValueToName(resVO.getType()));

                    QueryWrapper<IdenPersonCrowdRef> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.lambda().eq(IdenPersonCrowdRef::getPersonId,resVO.getId());
                    List<IdenPersonCrowdRef> listIdenPersonCrowdRef = idenPersonCrowdRefService.list(queryWrapper1);
                    StringBuilder sb = new StringBuilder();
                    if (CollUtil.isNotEmpty(listIdenPersonCrowdRef)) {
                       for(IdenPersonCrowdRef idenPersonCrowdRef : listIdenPersonCrowdRef){
                           IdenCrowd idenCrowd = idenCrowdService.getById(idenPersonCrowdRef.getCrowdId());
                           if(idenCrowd != null) {
                               sb.append(idenCrowd.getName()).append(",");
                           }
                       }
                    }
                    String crowdName = sb.toString();
                    if(crowdName != null && crowdName.endsWith(",")) {
                        crowdName = crowdName.substring(0, crowdName.length() - 1);
                    }
                    resVO.setCrowdName(crowdName);
                    list.add(resVO);
                });
                results.setRecords(list);
            }
            return results;
    }

    /**
     * 删除人员
     * @param id
     * @return
     */
    public boolean deleteById(Long id){
        return this.idenPersonService.removeById(id);
    }

    /**
     * 详情
     * @param id
     * @return
     */
    public PersonVO getPersonById(Long id){
        IdenPerson idenPerson = this.idenPersonService.getById(id);
        if (idenPerson!=null){
            PersonVO resVO = new PersonVO();
            BeanUtil.copyProperties(idenPerson,resVO);
            resVO.setPopulationTypeName(PopulationTypeEnum.getValueToName(resVO.getPopulationType()));
            resVO.setTypeName(PersonTypeEnum.getValueToName(resVO.getType()));

            QueryWrapper<IdenPersonCrowdRef> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.lambda().eq(IdenPersonCrowdRef::getPersonId,resVO.getId());
            List<IdenPersonCrowdRef> listIdenPersonCrowdRef = idenPersonCrowdRefService.list(queryWrapper1);
            StringBuilder sb = new StringBuilder();
            if (CollUtil.isNotEmpty(listIdenPersonCrowdRef)) {
                for(IdenPersonCrowdRef idenPersonCrowdRef : listIdenPersonCrowdRef){
                    IdenCrowd idenCrowd = idenCrowdService.getById(idenPersonCrowdRef.getCrowdId());
                    if(idenCrowd != null) {
                        sb.append(idenCrowd.getName()).append(",");
                    }
                }
            }
            String crowdName = sb.toString();
            if(crowdName != null && crowdName.endsWith(",")) {
                crowdName = crowdName.substring(0, crowdName.length() - 1);
            }
            resVO.setCrowdName(crowdName);
            return resVO;
        }
        return null;
    }

    /**
     * 上传图像
     * @param file
     * @return  访问URL
     */
    public String uploadImage(MultipartFile file) throws BDException {
        String image = null;
        try {
            if (file != null) {
                //获取文件名
                String fileName = file.getOriginalFilename();
                if (org.springframework.util.StringUtils.isEmpty(fileName) || file.getSize() == 0) {
                    throw new BDException("图像文件不能为空！");
                }
                //验证文件名是否合格
                if (!ImgUtil.isImg(fileName)) {
                    throw new BDException("图像文件必须是图片格式！");
                }
                String saveFileName = DateUtils.getCurrYyyyMMddHHmmssDate() + "_" + UUID.randomUUID().toString()  + fileName.substring(fileName.lastIndexOf("."), fileName.length());
                String picFullFileName = idenRoot + "data/final/person/" + saveFileName;
                FileOutputStream fos = new FileOutputStream(picFullFileName);
                fos.write(file.getBytes());
                image = fileUrl + "person/" + saveFileName;
            } else {
                throw new BDException("上传失败");
            }
        } catch (Exception e) {
            throw new BDException("上传失败",e);
        }

        return image;
    }


    /**
     * 保存人员
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public int createPerson(PersonVO vo, UserLoginedConvertVO loginUser){
        QueryWrapper<IdenPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(IdenPerson::getCredentialsCode,vo.getCredentialsCode());
        if(idenPersonService.count(queryWrapper) > 0){
            return 1;
        }
        //保存人员
        IdenPerson idenPerson = new IdenPerson();
        BeanUtil.copyProperties(vo,idenPerson);
        idenPerson.setCreateTime(new Date());
        //识别
        String image = idenPerson.getImage();
        if(StringUtils.isNotEmpty(image)){
            String fileName = image.substring(image.lastIndexOf("/"));
            File imgFile = new File(idenRoot + "data/final/person/" + fileName);
            String featPtr = FaceIdenTool.getFeatPtr(idenRoot,imgFile);
            idenPerson.setFeatPtr(featPtr);
        }

        this.idenPersonService.save(idenPerson);
        return 0;
    }

    /**
     * 修改人员
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public void updatePerson(PersonVO vo){
        //保存人员
        IdenPerson idenPerson = this.idenPersonService.getById(vo.getId());
        String oldImage = idenPerson.getImage();
        MyBeanUtils.copyProperties(vo,idenPerson);
        String newImage = vo.getImage();
        if (StringUtils.isNotEmpty(newImage) && !newImage.equals(oldImage)){
            //识别
            String fileNameNew = newImage.substring(newImage.lastIndexOf("/"));
            File imgFileNew = new File(idenRoot + "data/final/person/" + fileNameNew);
            String featPtr = FaceIdenTool.getFeatPtr(idenRoot,imgFileNew);
            idenPerson.setFeatPtr(featPtr);
            // 删除老的文件
            if(StringUtils.isNotEmpty(oldImage)){
                String fileNameOld = oldImage.substring(oldImage.lastIndexOf("/"));
                File imgFileOld = new File(idenRoot + "data/final/person/" + fileNameOld);
                if(imgFileOld.exists()){
                    imgFileOld.delete();
                }
            }

        }
        idenPerson.setModifyTime(new Date());

        this.idenPersonService.updateById(idenPerson);
    }

    /**
     * 根据图像文件名称得到IdenPerson
     * @param imageFileName
     * @return
     */
    public IdenPerson getPersonIdByImageFileName(String imageFileName){

        QueryWrapper<IdenPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(IdenPerson::getImage,imageFileName);
        IdenPerson idenPerson = idenPersonService.getOne(queryWrapper);
        return idenPerson;
    }
}